#!/usr/bin/python3
import sys, getopt, json

configuration = '/etc/wb-hardware.conf'

# default params
modem = 'absent'
zigbee = 'absent'
line_a = 'rs485'
line_b = 'rs485'

# acceptable params
modem_list = ["wb_4g", "wb_3g"]
zigbee_list = ["wb_cc2530"]
line_a_list = ["rs485"]
line_b_list = ["rs485", "uart"]

def get_board_revision():
    file_path = '/proc/device-tree/wirenboard/board-revision'

    try:
        with open(file_path, 'r') as file:
            first_char = file.read(1)
            return first_char
    except FileNotFoundError:
        print(f"Файл {file_path} не существует.")
        return None

def check_input(input_value, checking_list):
    if not input_value in checking_list:
        help()
        sys.exit(2)

def help():
    print('configure_hardware.py -c <configuration> -m <modem> -z <zigbee> -a <rs485> -b <rs485 | uart>')

def parse_args(argv):

    try:
        opts, args = getopt.getopt(argv,"hc:m:z:a:b:",["configuration=", "modem=", "zigbee=", "line_a=", "line_b=" ])
    except getopt.GetoptError:
        help()
        sys.exit(2)
    for opt, arg in opts:
        if opt == '-h':
            help()
            sys.exit()
        elif opt in ("-c", "--configuration"):
            global configuration
            configuration = arg
        elif opt in ("-m", "--modem"):
            check_input(arg, modem_list)
            global modem
            modem = arg
        elif opt in ("-z", "--zigbee"):
            check_input(arg, zigbee_list)
            global zigbee
            zigbee = arg
        elif opt in ("-a", "--line_a"):
            check_input(arg, line_a_list)
            global line_a
            line_a = arg
        elif opt in ("-b", "--line_b"):
            check_input(arg, line_b_list)
            global line_b
            line_b = arg

def set_zigbee(cfg_data):
    if zigbee == 'wb_cc2530':
        if 'slots' in cfg_data:
            for slot in cfg_data['slots']:
                if 'wb72-mod1' == slot['id']:
                    slot['module'] = 'wbe2r-r-zigbee'
                elif 'wb67-mod1' == slot['id']:
                    slot['module'] = 'wbe2r-r-zigbee'
                elif 'wb6-mod1' == slot['id']:
                    slot['module'] = 'wbe2r-r-zigbee'
        else:
            cfg_data['mod1'] = {'module': 'wbe2r-r-zigbee', 'options': {}}

def set_modem(cfg_data):
    revision = get_board_revision()

    if modem == 'wb_4g':
        if 'slots' in cfg_data:
            for slot in cfg_data['slots']:
                if 'wb67-wbc' == slot['id']:
                    slot['module'] = 'wbc-4g'
                elif 'wb72-wbc' == slot['id']:
                    slot['module'] = 'wbc-4g-usb'
        else:
            if revision == '6':
                cfg_data['wbc'] = {'module': 'wbc-4g', 'options': {}}
            elif revision == '7':
                cfg_data['wbc'] = {'module': 'wbc-4g-usb', 'options': {}}
            else:
                print(f"Ревизия \"{revision}\" не известна.")

    elif modem == 'wb_3g':
        if 'slots' in cfg_data:
            for slot in cfg_data['slots']:
                if 'wb67-wbc' == slot['id']:
                    slot['module'] = 'wbc-3g'
                elif 'wb72-wbc' == slot['id']:
                    slot['module'] = 'wbc-3g-usb'
        else:
            if revision == '6':
                cfg_data['wbc'] = {'module': 'wbc-3g', 'options': {}}
            elif revision == '7':
                cfg_data['wbc'] = {'module': 'wbc-3g-usb', 'options': {}}
            else:
                print(f"Ревизия \"{revision}\" не известна.")

def set_rs485_line_1(cfg_data):
    if 'slots' in cfg_data:
        for slot in cfg_data['slots']:
            if 'wb67-rs485-1' == slot['id']:
                slot['module'] = 'wb67-can-rs485'
            elif 'wb72-rs485-1' == slot['id']:
                slot['module'] = 'wb67-can-rs485'
            elif 'wb6-rs485-1' == slot['id']:
                slot['module'] = 'wb6-can-rs485'

def set_rs485_line_2(cfg_data):
    if 'slots' in cfg_data:
        for slot in cfg_data['slots']:
            if 'wb67-rs485-2' == slot['id']:
                slot['module'] = 'wb67-can-rs485'
            elif 'wb72-rs485-2' == slot['id']:
                slot['module'] = 'wb67-can-rs485'
            elif 'wb6-rs485-2' == slot['id']:
                slot['module'] = 'wb6-can-rs485'

def set_uart_line_2(cfg_data): # Only available for second line
    if 'slots' in cfg_data:
        for slot in cfg_data['slots']:
            if 'wb67-rs485-2' == slot['id']:
                slot['module'] = 'wb67-can-uart'
            elif 'wb72-rs485-2' == slot['id']:
                slot['module'] = 'wb67-can-uart'
            elif 'wb6-rs485-2' == slot['id']:
                slot['module'] = 'wb6-can-uart'
    else:
        cfg_data['rs485-2'] = {'module': 'wb67-can-uart', 'options': { 'terminatorsMode': 'disabled' }}


def configure_hardware():
    with open(configuration, 'r+') as f:
        cfg_data = json.load(f)

        set_zigbee(cfg_data)
        set_modem(cfg_data)

        set_rs485_line_1(cfg_data)

        if "uart" == line_b:
            set_uart_line_2(cfg_data)
        else:
            # modbus by default
            set_rs485_line_2(cfg_data)

        f.seek(0)        # <--- should reset file position to the beginning.
        json.dump(cfg_data, f, indent=4)
        f.truncate()     # remove remaining part

if __name__ == "__main__":
    parse_args(sys.argv[1:])
    print('Zigbee module is "{}"'.format(zigbee))
    print('Modem is "{}"'.format(modem))
    print('A line: "{}"'.format(line_a))
    print('B line: "{}"'.format(line_b))
    configure_hardware()
