#!/bin/bash

SCRIPT_DIR=$(dirname "${BASH_SOURCE[0]}")
PATH=$PATH:$SCRIPT_DIR

CONNECTION_MANAGER_CFG="/etc/wb-connection-manager.conf"

function upsert_wifi_connection() {
    local priority="$1"
    local con_name="$2"
    local ssid="$3"
    local password="$4"

    if [ -z "$priority" ] || [ -z "$con_name" ] || [ -z "$ssid" ] || [ -z "$password" ]; then
        echo "Usage: upsert_wifi_connection <priority> <con_name> <ssid> <password>"
        return 1
    fi

    # Проверка на наличие соединения с таким именем
    if ! nmcli connection show | grep -q "^$con_name\s"; then
        echo "Connection '$con_name' does not exist. Creating it..."
        nmcli connection add type wifi con-name "$con_name" ifname wlan1 ssid "$ssid"
    else
        echo "Connection '$con_name' already exists. Updating it..."
    fi

    nmcli connection modify "$con_name" wifi-sec.key-mgmt wpa-psk
    nmcli connection modify "$con_name" wifi-sec.psk "$password"

    update_priority.py "$con_name" "update" "$CONNECTION_MANAGER_CFG" "$priority"
}

function upsert_ethernet_connection() {
    local priority="$1"
    local con_name="$2"

    if [ -z "$priority" ] || [ -z "$con_name" ]; then
       echo "Usage: upsert_ethernet_connection <priority> <con_name>"
       return 1
   fi

    # Проверка на наличие соединения с таким именем
    if ! nmcli connection show | grep -q "^$con_name\s"; then
        echo "Connection '$con_name' does not exist. Creating it..."
        nmcli connection add type ethernet con-name "$con_name" ifname eth0
    else
        echo "Connection '$con_name' already exists. Updating it..."
    fi

    update_priority.py "$con_name" "update" "$CONNECTION_MANAGER_CFG" "$priority"
}

function upsert_gsm_connection() {
    local priority="$1"
    local con_name="$2"

    if [ -z "$priority" ] || [ -z "$con_name" ]; then
        echo "Usage: upsert_gsm_connection <priority> <con_name>"
        return 1
    fi

    # Проверка на наличие соединения с таким именем
    if ! nmcli connection show | grep -q "^$con_name\s"; then
        echo "Connection '$con_name' does not exist. Creating it..."
        nmcli connection add type gsm con-name "$con_name" ifname "*"
    else
        echo "Connection '$con_name' already exists. Updating it..."
    fi

    # Дополнительные настройки для GSM-соединения
    nmcli connection modify "$con_name" gsm.auto-config true
    nmcli connection modify "$con_name" ipv4.method auto
    nmcli connection modify "$con_name" ipv6.addr-gen-mode default
    nmcli connection modify "$con_name" ipv6.method auto

    update_priority.py "$con_name" "update" "$CONNECTION_MANAGER_CFG" "$priority"
}

function remove_connection() {
    local con_name="$1"

    if [ -z "$con_name" ]; then
        echo "Usage: remove_connection <con_name>"
        return 1
    fi

    # Проверка наличия соединения с таким именем
    if nmcli connection show | grep -q "^$con_name\s"; then
        echo "Removing connection '$con_name'..."
        nmcli connection delete "$con_name"
        echo "Connection '$con_name' removed."
    else
        echo "Connection '$con_name' does not exist."
    fi

    update_priority.py "$con_name" "remove" "$CONNECTION_MANAGER_CFG"
}

function wait_for_network() {
    local timeout_in_seconds="$1"
    local node_address="$2"
    local continuous_check_duration=10
    local success_count=0

    start_time=$(date +%s)

    while true; do
        if ping -c 1 -i 1 "$node_address" &>/dev/null; then
            ((success_count++))
            if [ $success_count -ge $continuous_check_duration ]; then
                echo "[$(date "+%Y-%m-%d %H:%M:%S")] Server is available for at least ${continuous_check_duration} seconds! Continuing..."
                return 0
            fi
        else
            success_count=0
        fi

        current_time=$(date "+%Y-%m-%d %H:%M:%S")
        elapsed_time=$(( $(date +%s) - start_time ))

        if [ $elapsed_time -ge $timeout_in_seconds ]; then
            echo "[${current_time}] Error! Network is not established within ${timeout_in_seconds} seconds. Exiting with an error."
            return 1
        fi

        echo "[${current_time}] Waiting for network..."
        sleep 1
    done
}

function update_priority() {
    local priority="$1"
    local con_name="$2"

    # Вызов Python скрипта, т.к. на bash с json работать не удобно
    update_priority.py "$2" "update" $CONNECTION_MANAGER_CFG "$1"
}

call_function() {
    local function_name=$1
    shift
    $function_name "$@"
}

if [[ "${BASH_SOURCE[0]}" == "$0" ]]; then
    if declare -f "$1" > /dev/null; then
        call_function "$@"
    else
        echo "Error: Function '$1' not found"
        exit 1
    fi
fi
