#!/bin/bash

function help() {
    cat <<EOF
This bash script upgrades Wirenboard devices over a Modbus connection.

Usage option 1st:
  wbfw_updater.sh --all

Options:
  --all        Upgrade all devices from /etc/wb-mqtt-serial.conf

Usage option 2nd:
  wbfw_updater.sh -a ADDRS -u BUS [-p PARITY] [-s SBITS] [-d DBITS] [-b BRATE] [-v VERSION] [-r BRANCH]

Options:
  -a ADDRS     Addresses of the devices on the bus, separated by commas.
  -u BUS       Bus name where the devices are located.
  -p PARITY    Parity (N/O/E) of the serial connection. Default is "N".
  -s SBITS     Number of stop bits. Default is 2.
  -d DBITS     Number of data bits. Default is 8.
  -b BRATE     Baud rate speed. Default is 9600.
  -v VERSION   Version of firmware to download. Default is "latest".
  -r BRANCH    Branch of firmware to download. Default is "main".

Examples:
  wbfw_updater.sh --all
  wbfw_updater.sh -a 1,2,3 -u /dev/ttyRS485-1
  wbfw_updater.sh -a 1,2,3 -u /dev/ttyRS485-1 -p N -s 2 -d 8 -b 9600 -v latest -r main
EOF
}

rand_string() {
    tr -dc 'a-zA-Z0-9' </dev/urandom | fold -w "${1:-32}" | head -n 1
}

TMP_DIR="/tmp/$(rand_string 20)"
mkdir -p "$TMP_DIR"

download_wbfw() {
    echo download_wbfw
    local signature
    local version
    local branch

    local max_attempts=5
    local attempt=0

    local OPTIND
    while getopts "t:v:r:" opt; do
        case $opt in
        t) signature="$OPTARG" ;;
        v) version="$OPTARG" ;;
        r) branch="$OPTARG" ;;
        \?) echo "Invalid option: -$OPTARG" >&2
            help
            exit 1
            ;;
        :)
            echo "Option -$OPTARG requires an argument." >&2
            help
            exit 1
            ;;
        esac
    done
    shift $((OPTIND - 1))

    if [ -z "$signature" ] || [ -z "$version" ] || [ -z "$branch" ]; then
        echo "Error: missing required argument for signature, version or branch" >&2
        help
        exit 1
    fi

    declare -g FIRMWARE="${TMP_DIR}/${branch}_${signature}_${version}.wbfw"

    if [ "${branch}" == "main" ]; then
        local url=https://fw-releases.wirenboard.com/fw/by-signature/"$signature"/main/${version}.wbfw
    else
        local url=https://fw-releases.wirenboard.com/fw/by-signature/"$signature"/unstable/${branch}/${version}.wbfw
    fi

    while [ ! -e "$FIRMWARE" ] && [ $attempt -lt $max_attempts ]; do
        curl -o "$FIRMWARE" "$url"
        ((attempt++))
        sleep 0.5
    done

    if [ ! -e "$FIRMWARE" ]; then
        echo "Failed to download $FIRMWARE after $attempt attempts." >&2
        return 1
    fi

    return 0
}

get_signature() {
    echo get_signature
    local parity="none"
    local stop_bits=2
    local data_bits=8
    local baud_rate=9600
    local slave_id
    local path

    local max_attempts=5
    local attempt=0

    local OPTIND
    while getopts "p:s:d:b:a:u:" opt; do
        case $opt in
        p) parity="$OPTARG" ;;
        s) stop_bits="$OPTARG" ;;
        d) data_bits="$OPTARG" ;;
        b) baud_rate="$OPTARG" ;;
        a) slave_id="$OPTARG" ;;
        u) path="$OPTARG" ;;
        \?) echo "Invalid option: -$OPTARG" >&2
            help
            exit 1
            ;;
        :)
            echo "Option -$OPTARG requires an argument." >&2
            exit 1
            ;;
        esac
    done
    shift $((OPTIND - 1))

    until
        signature=$(echo -e $(modbus_client -mrtu -p${parity} -d${data_bits} -s${stop_bits} -b${baud_rate} "${path}" -a ${slave_id} -t3 -r 290 -c 12 | grep Data | sed -e 's/.*Data://' -e 's/ 0x00/\\x/g') | tr '\0' '\n')
        [ -n "$signature" ]
    do
        : sleep 0.1
        ((attempt++))
        echo "Device located on ${path} with address ${slave_id} not found. Try again..."

        if [ $attempt -gt $max_attempts ]; then
            echo "Device located on ${path} with address ${slave_id} not found. Skip it."
            break
        fi
    done

    declare -g SIGNATURE="${signature}"
}

upgrade_device() {
    echo upgrade_device
    local parity="none"
    local stop_bits=2
    local data_bits=8
    local baud_rate=9600
    local -a slave_ids=() # Array for storing addresses
    local path
    local version="latest"
    local branch="main"

    local OPTIND
    while getopts "p:s:d:b:a:u:v:r:" opt; do
        case $opt in
        p) case "$OPTARG" in
            N) parity=none ;;
            O) parity=odd ;;
            E) parity=even ;;
            /?) echo "Invalid option: -$OPTARG" >&2
                help
                exit 1
                ;;
            esac ;;
        s) stop_bits="$OPTARG" ;;
        d) data_bits="$OPTARG" ;;
        b) baud_rate="$OPTARG" ;;
        a) IFS=',' read -ra slave_ids <<< "$OPTARG" ;; # Read addresses into an array, separated by commas
        u) path="$OPTARG" ;;
        r) branch="$OPTARG" ;;
        v) version="$OPTARG" ;;
        \?) echo "Invalid option: -$OPTARG" >&2
            help
            exit 1
            ;;
        :)
            echo "Option -$OPTARG requires an argument." >&2
            help
            exit 1
            ;;
        esac
    done
    shift $((OPTIND - 1))

    for slave_id in "${slave_ids[@]}"; do
        if [ -z "$slave_id" ] || [ -z "$path" ]; then
            echo "Error: missing required argument for address or path" >&2
            help
            exit 1
        fi

        SIGNATURE=
        get_signature -p "$parity" -s "$stop_bits" -d "$data_bits" -b "$baud_rate" -a "$slave_id" -u "$path"

        if [ -z "$SIGNATURE" ]; then
            echo "Signature not found for device at address $slave_id"
            continue
        fi

        download_wbfw -t "$SIGNATURE" -v "$version" -r "$branch" || continue

        echo
        echo "Upgrading device at address $slave_id:"
        echo "Signature      ${SIGNATURE}"
        echo "Parity         ${parity}"
        echo "Stop bits      ${stop_bits}"
        echo "Data bits      ${data_bits}"
        echo "Baud rate      ${baud_rate}"
        echo "Path           ${path}"
        echo "Target version ${version}"

        wb-mcu-fw-flasher -j -d "${path}" -a "${slave_id}" -f "${FIRMWARE}"
    done
}

upgrade_all() {
    upgraded=()
    fault=()

    while read i; do
        baud_rate=$(echo $i | jq -r '.baud_rate')
        parity=$(echo $i | jq -r '.parity')
        data_bits=$(echo $i | jq -r '.data_bits')
        stop_bits=$(echo $i | jq -r '.stop_bits')
        path=$(echo $i | jq -r '.path')

        while read j; do
            slave_id=$(echo $j | jq -r '.slave_id')
            device_type=$(echo $j | jq -r '.device_type')
            echo "Working on ${device_type}_${slave_id}"
            upgrade_device -b "${baud_rate}" -p "${parity}" -d "${data_bits}" -s "${stop_bits}" -u "${path}" -a "${slave_id}" &&
            upgraded+=( "${path}; ${device_type}; ${slave_id}" ) ||
            fault+=( "${path}; ${device_type}; ${slave_id}" )
        done <<< "$(echo $i | jq -c '.devices[]')"
    done <<< "$(jq -c '.ports[]' /etc/wb-mqtt-serial.conf)"

    local yc='\033[1;33m'
    local gc='\033[1;32m'
    local nc='\033[0m'

    printf "\n${nc}Successfully upgraded devices: ${gc}\n"
    for item in "${upgraded[@]}"; do
        echo "$item"
    done

    printf "\n${nc}Something went wrong with: ${yc}\n"
    for item in "${fault[@]}"; do
        echo "$item"
    done

    printf "${nc}"
}

if [[ $# -eq 0 ]] || [[ "$1" == "--help" ]] || [[ "$1" == "-h" ]]; then
    help
elif [[ "--all" == "$1" ]]; then
    upgrade_all
else
    upgrade_device "$@"
fi

rm -rf "$TMP_DIR"
